import requests
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)

from .app_settings import BASE_URL
from .provider import FyleProvider


class FyleOAuth2Adapter(OAuth2Adapter):
    """
    Fyle OAuth2Adapter
    """
    provider_id = FyleProvider.id
    access_token_url = '{0}/oauth/token'.format(BASE_URL)
    authorize_url = '{0}/app/developers/#/oauth/authorize'.format(BASE_URL)
    profile_url = '{0}/api/tpa/v1/employees/my_profile'.format(BASE_URL)

    def complete_login(self, request, app, token, **kwargs):
        """
        Returns a SocialLogin instance
        """
        headers = {'Authorization': 'Bearer {0}'.format(token.token)}
        resp = requests.get(self.profile_url, headers=headers)
        extra_data = resp.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(FyleOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(FyleOAuth2Adapter)



"""
django-allauth provider.py
"""

from allauth.socialaccount import providers
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class FyleAccount(ProviderAccount):
    pass


class FyleProvider(OAuth2Provider):
    """
    Fyle Provider class
    """
    id = 'fyle'
    name = 'Fyle'
    account_class = FyleAccount

    def extract_uid(self, data):
        """
        Extracts the unique user ID from `data`
        """
        return str(data['data']['user_id'])

    def extract_common_fields(self, data):
        """
        Extracts fields from `data` that will be used to populate the
        `User` model in the `SOCIALACCOUNT_ADAPTER`'s `populate_user()`
        method.

        For example:

            {'first_name': 'John'}

        :return: dictionary of key-value pairs.
        """
        return dict(email=data['data'].get('employee_email'),
                    username=data['data'].get('employee_email'),
                    name=data['data'].get('full_name'))

    def get_default_scope(self):
        """
        Get the OAuth scopes
        :return: list of scopes
        """
        scope = ['read', 'write', ]
        return scope


providers.registry.register(FyleProvider)
provider_classes = [FyleProvider]



import os
import re

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


def get_version(*file_paths):
    """Retrieves the version from fyle_django_allauth/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


setup(
    name='fyle-django-allauth',
    version=get_version("fyle_allauth", "__init__.py"),
    packages=find_packages(),
    include_package_data=True,
    install_requires=['Django >= 1.11', "django-allauth"],
    license='BSD License',  # example license
    description='Fyle OAuth2 provider for Django, lets you associate Fyle accounts with your User accounts.',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/fylein/fyle-django-allauth',
    author='akshay_codemonk',
    author_email='akshay@codemonk.in',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
    ],
)
